#! vtkpython

# This module allows you to extract data from a netCDF file
# and create a vtkDataSet out of it. You can create either
# a vtkStructuredPoints set, or a vtkRectilinearGrid.
# This module also contains the function 'view', which
# takes a 2D array, creates a graymap image from the data,
# and launches a viewer
#
# Usage (from within vtkpython):
#
# from nc2vtk import *
# import vtk
#
# c=CMesoFile("/bd/tornado/july02/cmeso20b.h07000.nc")
# qc=c['QC']
# view(qc[:,:,45])
# sp,raw=VTKStructuredPoints(qc)
# mc=vtk.vtkMarchingCubes()
# mc.SetInput(sp)
# ...
# rg,raw2=VTKRectilinearGrid(c['XCNTR'],c['YCNTR'],c['ZCNTR'],qc)
# ...

import rgbimg
import os
import tempfile

from Scientific.IO.NetCDF import *
import Numeric as Nm
import vtk

def view(inp):
    viewer=os.getenv("IMGVIEWER","display")
    if (Nm.rank(inp)!=2):
        raise("Can only view 2-d arrays!")
    ar=Nm.transpose(inp)
    mn=Nm.minimum.reduce(Nm.ravel(ar))
    mx=Nm.maximum.reduce(Nm.ravel(ar))
    nr=((Nm.ravel(ar)-mn)/(mx-mn)*255).astype('b')
    u=nr*Nm.array(0x01000000,'u')+nr*0x00010100
    nm=tempfile.mktemp('.rgb')
    rgbimg.longstoimage((u).tostring(),ar.shape[1],ar.shape[0],1,nm)
    os.spawnlp(os.P_NOWAIT,viewer,"",nm)

def TypeCodeToVTKArray(tc):
    if   tc=='d': return vtk.vtkDoubleArray()
    elif tc=='f': return vtk.vtkFloatArray()
    elif tc=='l': return vtk.vtkIntArray()
    elif tc=='1': return vtk.vtkCharArray()
    elif tc=='s': return vtk.vtkShortArray()
    elif tc=='i': return vtk.vtkIntArray()
    else: raise("No such type!")

    

class CMesoFile:
    def __init__(self,fname,rename_map=None):
        """Rename_map is a dict mapping alternate names to their canonical form
        """
        self.f=NetCDFFile(fname)
        self.datas={}
        self.strs={}
        if rename_map==None: rename_map={}
        self.canon_to_local={}
        for var in self.f.variables:
            canon=rename_map.get(var,var)
            self.canon_to_local[canon]=var

    def __del__(self):
        self.f.close()
        
    def __getitem__(self,index):
        localname=self.canon_to_local[index]
        if self.datas.has_key(localname):
            return self.datas[localname]
        if self.f.variables.has_key(localname):
            v=self.f.variables[localname].getValue()
            self.datas[localname]=v
            return v
        raise("File contains no such variable: %s." %localname)

    def missing_value(self,varname):
        localname=self.canon_to_local[varname]
        if self.f.variables.has_key(localname):
            v=self.f.variables[localname]
            if hasattr(v,"missing_value"):
                return v.missing_value
        return None
        
    def GetVoidData(self,index):
        localname=self.canon_to_local[index]
        if self.strs.has_key(localname):
            return self.strs[localname]
        # Export the data in C array ordering...
        self.strs[localname]=Nm.transpose(self[localname]).tostring()
        return self.strs[localname]
                         

def GetVTKArray(nc_array,fortranorder=True):
    if fortranorder:
        raw=Nm.transpose(nc_array).tostring()
    else:
        raw=nc_array.tostring()
    result=TypeCodeToVTKArray(nc_array.typecode())
    nelems=Nm.size(nc_array) 
    result.SetNumberOfValues(nelems)
    result.SetVoidArray(raw, nelems,1)
    return result,raw


def VTKStructuredPoints(nc_array,fortranorder=True):
    result=vtk.vtkStructuredPoints()
    shp=nc_array.shape
    if len(shp)==2:
	shp=(shp[0],shp[1],1)
    if (not fortranorder):
        shp=shp[::-1]
    
    result.SetDimensions(shp)
    array,raw=GetVTKArray(nc_array,fortranorder)
    result.GetPointData().SetScalars(array)
    return result,raw

def VTKRectilinearGrid(xc,yc,zc,nc_array,fortranorder=True):
    result=vtk.vtkRectilinearGrid()
    shp=nc_array.shape
    if len(shp)==2:
        result.SetDimensions(shp[0],shp[1],1)
    else:
   	result.SetDimensions(shp)
    xdata,xraw=GetVTKArray(xc)
    result.SetXCoordinates(xdata)
    
    ydata,yraw=GetVTKArray(yc)
    result.SetYCoordinates(ydata)
    
    if len(shp)==3:
	zdata,zraw=GetVTKArray(zc)
        result.SetZCoordinates(zdata)
    else:
	zraw=0
    
    array,raw=GetVTKArray(nc_array,fortranorder)
    result.GetPointData().SetScalars(array)
    return result,(raw,xraw,yraw,zraw)

def AddTextures(ds,u,v=None,fortranorder=True):
    dims=ds.GetDimensions()
    sz=reduce(lambda x,y:x*y,dims,1)
    if ((u and u.shape!=dims) or (v and v.shape!=dims)):
        print >>sys.stderr,"Texture data has improper extents"
        return
    arr=Nm.zeros(2*sz,Nm.Float32)
    if fortranorder:
        if u:
            arr[::2]=Nm.ravel(Nm.transpose(u.astype(Nm.Float32)))
        if v:
            arr[1::2]=Nm.ravel(Nm.transpose(v.astype(Nm.Float32)))
    else:
        if u:
            arr[::2]=Nm.ravel(u.astype(Nm.Float32))
        if v:
            arr[1::2]=Nm.ravel(v.astype(Nm.Float32))
    ds.raw_tc=arr.tostring()
    tc=vtk.vtkFloatArray()
    tc.SetNumberOfComponents(2)
    tc.SetNumberOfTuples(sz)
    tc.SetVoidArray(ds.raw_tc,sz,1)
    ds.GetPointData().SetTCoords(tc)
    
def AddFieldData(ds,scalar_data,scalar_name,scalar_type=Nm.Float32,fortranorder=True):
    dims=ds.GetDimensions()
    sz=reduce(lambda x,y:x*y,dims,1)
    if (scalar_data.shape!=dims):
        print >>sys.stderr,"Scalar data has improper extents"
        return
    if fortranorder:
        arrdat=Nm.ravel(Nm.transpose(scalar_data.astype(scalar_type))).tostring()
    else:
        arrdat=Nm.ravel(scalar_data.astype(scalar_type)).tostring()
    if not hasattr(ds,"raw_scalars"):
        ds.raw_scalars={}
    ds.raw_scalars[scalar_name]=arrdat

    sarr=vtk.vtkFloatArray()
    sarr.SetNumberOfTuples(sz)
    sarr.SetVoidArray(arrdat,sz,1)
    sarr.SetName(scalar_name)
    ds.GetPointData().AddArray(sarr)



# compute a python expression of arrays in a netcdf file
# Also allow any symbols from the dict other_symbols
def ParseArrayExpr(ncf,expr,other_symbols={}):
    arr_data=other_symbols.copy()
    #only load in the datasets used in the expression
    for key in ncf.canon_to_local.keys():
        if expr.find(key)>=0:
            arr_data[key]=ncf[key]
    return eval(expr,Nm.__dict__,arr_data)


        
def CMesoRG(fname="../july02/cmeso20b.h06850.nc",vname="QC"):
  cmf=CMesoFile(fname)
  return VTKRectilinearGrid(cmf["XCNTR"]/1000-50,cmf["YCNTR"]/1000-50,
cmf["ZCNTR"]/1000,cmf[vname])

# Multiple threads cause seams in surfaces. blech
vtk.vtkMultiThreader().SetGlobalDefaultNumberOfThreads(1)



if __name__ == "__main__":
  import sys

  if len(sys.argv)<3:
    print "Usage: nc2vtk ncfile varname [outname]"
    print "Outputs to ncfile.varname.vtk or outname.vtk"

  ncfile=sys.argv[1]
  varname=sys.argv[2]
  if len(sys.argv)==4:
    outname=sys.argv[3]+"."+varname+".vtk"
  else:
    outname=sys.argv[2]+"."+varname+".vtk"

  C=CMesoFile(ncfile)
  vdata,raw=VTKStructuredPoints(C[varname])
  out=vtk.vtkStructuredPointsWriter()
  out.SetInput(vdata)
  out.SetFileName(outname)
  out.SetFileTypeToBinary()
  out.Update()
